CA_TEMP_RANGE_ICE_C = [x * 0.5 for x in range(32, 64)]
CA_TEMP_RANGE_EV_C = [x * 0.5 for x in range(28, 64)]

SENSITIVE_FIELD_NAMES = [
    "username",
    "password",
    "userid",
    "vin",
    "sid",
    "vinkey",
    "lat",
    "lon",
]
